@@ -23,7 +23,6 @@
 #include <config.h>
 #endif
 
-#define _GNU_SOURCE
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -66,15 +65,17 @@ static const char *oercn_prefix[] = { "_OERCN:", NULL };
 static const char *cpinr_prefixes[] = { "+CPINR:", "+CPINRE:", NULL };
 static const char *epin_prefix[] = { "*EPIN:", NULL };
 static const char *simcom_spic_prefix[] = { "+SPIC:", NULL };
-static const char *cinterion_spic_prefix[] = { "^SPIC:", NULL };
+static const char *gemalto_spic_prefix[] = { "^SPIC:", NULL };
 static const char *pct_prefix[] = { "#PCT:", NULL };
 static const char *pnnm_prefix[] = { "+PNNM:", NULL };
 static const char *qpinc_prefix[] = { "+QPINC:", NULL };
+static const char *qtrpin_prefix[] = { "+QTRPIN:", NULL };
 static const char *upincnt_prefix[] = { "+UPINCNT:", NULL };
 static const char *cuad_prefix[] = { "+CUAD:", NULL };
 static const char *ccho_prefix[] = { "+CCHO:", NULL };
 static const char *crla_prefix[] = { "+CRLA:", NULL };
 static const char *cgla_prefix[] = { "+CGLA:", NULL };
+static const char *xcmscsc_prefix[] = { "+XCMSCSC:", NULL};
 static const char *none_prefix[] = { NULL };
 
 static void append_file_path(char *buf, const unsigned char *path,
@@ -982,6 +983,49 @@ error:
 	CALLBACK_WITH_FAILURE(cb, NULL, cbd->data);
 }
 
+static void at_qtrpin_cb(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct cb_data *cbd = user_data;
+	ofono_sim_pin_retries_cb_t cb = cbd->cb;
+	const char *final = g_at_result_final_response(result);
+	GAtResultIter iter;
+	struct ofono_error error;
+	int retries[OFONO_SIM_PASSWORD_INVALID];
+	size_t i;
+
+	decode_at_error(&error, final);
+
+	if (!ok) {
+		cb(&error, NULL, cbd->data);
+		return;
+	}
+
+	for (i = 0; i < OFONO_SIM_PASSWORD_INVALID; i++)
+		retries[i] = -1;
+
+	g_at_result_iter_init(&iter, result);
+
+	while (g_at_result_iter_next(&iter, "+QTRPIN:")) {
+		int pin, pin2, puk, puk2;
+
+		if (!g_at_result_iter_next_number(&iter, &pin))
+			continue;
+		if (!g_at_result_iter_next_number(&iter, &pin2))
+			continue;
+		if (!g_at_result_iter_next_number(&iter, &puk))
+			continue;
+		if (!g_at_result_iter_next_number(&iter, &puk2))
+			continue;
+
+		retries[OFONO_SIM_PASSWORD_SIM_PIN] = pin;
+		retries[OFONO_SIM_PASSWORD_SIM_PUK] = puk;
+		retries[OFONO_SIM_PASSWORD_SIM_PIN2] = pin2;
+		retries[OFONO_SIM_PASSWORD_SIM_PUK2] = puk2;
+	}
+
+	cb(&error, retries, cbd->data);
+}
+
 static void at_qpinc_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
@@ -1066,7 +1110,7 @@ error:
 	CALLBACK_WITH_FAILURE(cb, NULL, cbd->data);
 }
 
-static void cinterion_spic_cb(gboolean ok, GAtResult *result,
+static void gemalto_spic_cb(gboolean ok, GAtResult *result,
 							gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
@@ -1117,6 +1161,7 @@ static void at_pin_retries_query(struct ofono_sim *sim,
 	DBG("");
 
 	switch (sd->vendor) {
+	case OFONO_VENDOR_XMM:
 	case OFONO_VENDOR_IFX:
 		if (g_at_chat_send(sd->chat, "AT+XPINCNT", xpincnt_prefix,
 					xpincnt_cb, cbd, g_free) > 0)
@@ -1172,15 +1217,19 @@ static void at_pin_retries_query(struct ofono_sim *sim,
 					at_qpinc_cb, cbd, g_free) > 0)
 			return;
 		break;
+	case OFONO_VENDOR_QUECTEL_SERIAL:
+		if (g_at_chat_send(sd->chat, "AT+QTRPIN", qtrpin_prefix,
+					at_qtrpin_cb, cbd, g_free) > 0)
+			return;
+		break;
 	case OFONO_VENDOR_UBLOX:
-	case OFONO_VENDOR_UBLOX_TOBY_L2:
 		if (g_at_chat_send(sd->chat, "AT+UPINCNT", upincnt_prefix,
 					upincnt_cb, cbd, g_free) > 0)
 			return;
 		break;
-	case OFONO_VENDOR_CINTERION:
-		if (g_at_chat_send(sd->chat, "AT^SPIC", cinterion_spic_prefix,
-					cinterion_spic_cb, cbd, g_free) > 0)
+	case OFONO_VENDOR_GEMALTO:
+		if (g_at_chat_send(sd->chat, "AT^SPIC", gemalto_spic_prefix,
+					gemalto_spic_cb, cbd, g_free) > 0)
 			return;
 		break;
 	default:
@@ -1305,6 +1354,7 @@ static void at_pin_send_cb(gboolean ok, GAtResult *result,
 	case OFONO_VENDOR_HUAWEI:
 	case OFONO_VENDOR_SIMCOM:
 	case OFONO_VENDOR_SIERRA:
+	case OFONO_VENDOR_QUECTEL_SERIAL:
 		/*
 		 * On ZTE modems, after pin is entered, SIM state is checked
 		 * by polling CPIN as their modem doesn't provide unsolicited
@@ -1872,6 +1922,83 @@ static void at_logical_access(struct ofono_sim *sim, int session_id,
 	CALLBACK_WITH_FAILURE(cb, NULL, 0, data);
 }
 
+static void xcmscsc_query_cb(gboolean ok, GAtResult *result, gpointer user)
+{
+	struct ofono_sim *sim = user;
+	struct sim_data *sd = ofono_sim_get_data(sim);
+	GAtResultIter iter;
+	int active_slot;
+
+	if (!ok)
+		goto done;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+XCMSCSC:"))
+		goto done;
+
+	g_at_result_iter_skip_next(&iter);
+	g_at_result_iter_skip_next(&iter);
+
+	g_at_result_iter_next_number(&iter, &active_slot);
+
+	/* set active SIM slot */
+	ofono_sim_set_active_card_slot(sim, active_slot + 1);
+
+done:
+	/* Query supported <fac>s */
+	g_at_chat_send(sd->chat, "AT+CLCK=?", clck_prefix,
+				at_clck_query_cb, sim, NULL);
+}
+
+static void at_xcmscsc_test_cb(gboolean ok, GAtResult *result, gpointer user)
+{
+	struct ofono_sim *sim = user;
+	struct sim_data *sd = ofono_sim_get_data(sim);
+	GAtResultIter iter;
+	int card_slot_count;
+
+	if (!ok)
+		goto done;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+XCMSCSC:"))
+		goto done;
+
+	g_at_result_iter_skip_next(&iter);
+	g_at_result_iter_skip_next(&iter);
+
+	if (!g_at_result_iter_open_list(&iter))
+		goto done;
+
+	g_at_result_iter_skip_next(&iter);
+
+	if (!g_at_result_iter_next_number(&iter, &card_slot_count))
+		goto done;
+
+	/* Set num slots */
+	ofono_sim_set_card_slot_count(sim, card_slot_count + 1);
+
+	/*
+	 * enable reporting of MSIM remap status information
+	 * and enable automatic acceptance of MSIM Remap
+	 * acknowledgement
+	 */
+	g_at_chat_send(sd->chat, "AT+XCMSRS=2", none_prefix,
+					NULL, NULL, NULL);
+
+	/* Query active card slot */
+	g_at_chat_send(sd->chat, "AT+XCMSCSC?", xcmscsc_prefix,
+				xcmscsc_query_cb, sim, NULL);
+	return;
+
+done:
+	/* Query supported <fac>s */
+	g_at_chat_send(sd->chat, "AT+CLCK=?", clck_prefix,
+				at_clck_query_cb, sim, NULL);
+}
+
 static int at_sim_probe(struct ofono_sim *sim, unsigned int vendor,
 				void *data)
 {
@@ -1890,6 +2017,10 @@ static int at_sim_probe(struct ofono_sim *sim, unsigned int vendor,
 		if (at_clck_cpwd_fac[i])
 			sd->passwd_type_mask |= (1 << i);
 
+	if (sd->vendor == OFONO_VENDOR_XMM)
+		return g_at_chat_send(sd->chat, "AT+XCMSCSC=?", xcmscsc_prefix,
+			at_xcmscsc_test_cb, sim, NULL) ? 0 : -1;
+
 	/* Query supported <fac>s */
 	return g_at_chat_send(sd->chat, "AT+CLCK=?", clck_prefix,
 				at_clck_query_cb, sim, NULL) ? 0 : -1;
@@ -1909,7 +2040,47 @@ static void at_sim_remove(struct ofono_sim *sim)
 	g_free(sd);
 }
 
-static struct ofono_sim_driver driver = {
+static void xcmscsc_cb(gboolean ok, GAtResult *result,
+		gpointer user_data)
+{
+	struct cb_data *cbd = user_data;
+	ofono_sim_set_active_card_slot_cb_t cb = cbd->cb;
+	struct ofono_error error;
+
+	decode_at_error(&error, g_at_result_final_response(result));
+
+	if (cb)
+		cb(&error, cbd->data);
+}
+
+static void at_set_active_card_slot(struct ofono_sim *sim, unsigned int index,
+			ofono_sim_set_active_card_slot_cb_t cb, void *data)
+{
+	struct sim_data *sd = ofono_sim_get_data(sim);
+	struct cb_data *cbd = cb_data_new(cb, data);
+	char cmd[43];
+
+	if (sd->vendor != OFONO_VENDOR_XMM) {
+		struct ofono_error error;
+		error.type = OFONO_ERROR_TYPE_CME;
+		error.error = 4;
+
+		cb(&error, data);
+		return;
+	}
+
+	/* Enable single SIM mode for indicated card slot id */
+	snprintf(cmd, sizeof(cmd), "AT+XCMSCSC=1,0,%u,1", index);
+
+	if (g_at_chat_send(sd->chat, cmd, none_prefix, xcmscsc_cb,
+			cbd, g_free) > 0)
+		return;
+
+	g_free(cbd);
+	CALLBACK_WITH_FAILURE(cb, data);
+}
+
+static const struct ofono_sim_driver driver = {
 	.name			= "atmodem",
 	.probe			= at_sim_probe,
 	.remove			= at_sim_remove,
@@ -1934,10 +2105,11 @@ static struct ofono_sim_driver driver = {
 	.session_read_binary	= at_session_read_binary,
 	.session_read_record	= at_session_read_record,
 	.session_read_info	= at_session_read_info,
-	.logical_access		= at_logical_access
+	.logical_access		= at_logical_access,
+	.set_active_card_slot	= at_set_active_card_slot
 };
 
-static struct ofono_sim_driver driver_noef = {
+static const struct ofono_sim_driver driver_noef = {
 	.name			= "atmodem-noef",
 	.probe			= at_sim_probe,
 	.remove			= at_sim_remove,
