@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
+ *  Copyright (C) 2010  Nokia Corporation and/or its subsidiary(-ies).
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -23,7 +23,6 @@
 #include <config.h>
 #endif
 
-#define _GNU_SOURCE
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -67,10 +66,12 @@ static void isi_call_verify_cb(const GIsiMessage *msg, void *data)
 	struct ofono_audio_settings *as = data;
 	struct audio_settings_data *asd = ofono_audio_settings_get_data(as);
 
-	if (g_isi_msg_error(msg) < 0)
+	if (g_isi_msg_error(msg) < 0) {
+		ofono_audio_settings_remove(as);
 		return;
+	}
 
-	ISI_VERSION_DBG(msg);
+	ISI_RESOURCE_DBG(msg);
 
 	g_isi_client_ind_subscribe(asd->client, CALL_SERVER_STATUS_IND,
 					isi_call_server_status_ind_cb,
@@ -115,7 +116,7 @@ static void isi_audio_settings_remove(struct ofono_audio_settings *as)
 	g_free(asd);
 }
 
-static struct ofono_audio_settings_driver driver = {
+static const struct ofono_audio_settings_driver driver = {
 	.name		= "isimodem",
 	.probe		= isi_audio_settings_probe,
 	.remove		= isi_audio_settings_remove,
