@@ -96,32 +96,7 @@ static void novatel_debug(const char *str, void *user_data)
 static GAtChat *open_device(struct ofono_modem *modem,
 						const char *key, char *debug)
 {
-	GAtChat *chat;
-	GAtSyntax *syntax;
-	GIOChannel *channel;
-	const char *device;
-
-	device = ofono_modem_get_string(modem, key);
-	if (device == NULL)
-		return NULL;
-
-	channel = g_at_tty_open(device, NULL);
-	if (channel == NULL)
-		return NULL;
-
-	syntax = g_at_syntax_new_gsm_permissive();
-	chat = g_at_chat_new(channel, syntax);
-	g_at_syntax_unref(syntax);
-
-	g_io_channel_unref(channel);
-
-	if (chat == NULL)
-		return NULL;
-
-	if (getenv("OFONO_AT_DEBUG"))
-		g_at_chat_set_debug(chat, novatel_debug, debug);
-
-	return chat;
+	return at_util_open_device(modem, key, novatel_debug, debug, NULL);
 }
 
 static void cfun_enable(gboolean ok, GAtResult *result, gpointer user_data)
