@@ -271,6 +271,9 @@ static gboolean assign_context(struct pri_context *ctx, unsigned int use_cid)
 		return TRUE;
 	}
 
+	l_uintset_take(used_cids, ctx->context.cid);
+	ctx->context.cid = 0;
+
 	return FALSE;
 }
 
@@ -579,7 +582,7 @@ static void pri_ifupdown(const char *interface, ofono_bool_t active)
 		return;
 
 	memset(&ifr, 0, sizeof(ifr));
-	strncpy(ifr.ifr_name, interface, IFNAMSIZ);
+	l_strlcpy(ifr.ifr_name, interface, IFNAMSIZ);
 
 	if (ioctl(sk, SIOCGIFFLAGS, &ifr) < 0)
 		goto done;
@@ -615,7 +618,7 @@ static void pri_set_ipv4_addr(const char *interface, const char *address)
 		return;
 
 	memset(&ifr, 0, sizeof(ifr));
-	strncpy(ifr.ifr_name, interface, IFNAMSIZ);
+	l_strlcpy(ifr.ifr_name, interface, IFNAMSIZ);
 
 	if (ioctl(sk, SIOCGIFFLAGS, &ifr) < 0)
 		goto done;
@@ -719,9 +722,7 @@ static void pri_update_mms_context_settings(struct pri_context *ctx)
 	struct ofono_gprs_context *gc = ctx->context_driver;
 	struct context_settings *settings = gc->settings;
 
-	if (ctx->message_proxy)
-		settings->ipv4->proxy = g_strdup(ctx->message_proxy);
-
+	settings->ipv4->proxy = g_strdup(ctx->message_proxy);
 	pri_parse_proxy(ctx, ctx->message_proxy);
 
 	DBG("proxy %s port %u", ctx->proxy_host, ctx->proxy_port);
@@ -1135,7 +1136,7 @@ static DBusMessage *pri_set_message_proxy(struct pri_context *ctx,
 	if (strlen(proxy) > MAX_MESSAGE_PROXY_LENGTH)
 		return __ofono_error_invalid_format(msg);
 
-	if (ctx->message_proxy && g_str_equal(ctx->message_proxy, proxy))
+	if (g_str_equal(ctx->message_proxy, proxy))
 		return dbus_message_new_method_return(msg);
 
 	strcpy(ctx->message_proxy, proxy);
@@ -1164,7 +1165,7 @@ static DBusMessage *pri_set_message_center(struct pri_context *ctx,
 	if (strlen(center) > MAX_MESSAGE_CENTER_LENGTH)
 		return __ofono_error_invalid_format(msg);
 
-	if (ctx->message_center && g_str_equal(ctx->message_center, center))
+	if (g_str_equal(ctx->message_center, center))
 		return dbus_message_new_method_return(msg);
 
 	strcpy(ctx->message_center, center);
@@ -1663,6 +1664,15 @@ static void gprs_netreg_update(struct ofono_gprs *gprs)
 {
 	ofono_bool_t attach;
 
+	/*
+	 * This function can get called by other reasons than netreg
+	 * updating its status. So check if we have a valid netreg status yet.
+	 * The only reason for not having a valid status is basically during
+	 * startup while the netreg atom is fetching the status.
+	 */
+	if (gprs->netreg_status < 0)
+		return;
+
 	attach = gprs->netreg_status == NETWORK_REGISTRATION_STATUS_REGISTERED;
 
 	attach = attach || (gprs->roaming_allowed &&
@@ -1700,7 +1710,7 @@ static void netreg_status_changed(int status, int lac, int ci, int tech,
 {
 	struct ofono_gprs *gprs = data;
 
-	DBG("%d", status);
+	DBG("%d (%s)", status, registration_status_to_string(status));
 
 	gprs->netreg_status = status;
 
@@ -1793,7 +1803,7 @@ static DBusMessage *gprs_set_property(DBusConnection *conn,
 		gprs->roaming_allowed = value;
 
 		if (gprs->settings) {
-			g_key_file_set_integer(gprs->settings, SETTINGS_GROUP,
+			g_key_file_set_boolean(gprs->settings, SETTINGS_GROUP,
 						"RoamingAllowed",
 						gprs->roaming_allowed);
 			storage_sync(gprs->imsi, SETTINGS_STORE,
@@ -1890,7 +1900,7 @@ static struct pri_context *find_usable_context(struct ofono_gprs *gprs,
 	for (l = gprs->contexts; l; l = l->next) {
 		pri_ctx = l->data;
 
-		if (pri_ctx->context.apn == NULL)
+		if (pri_ctx->context.apn[0] == '\0')
 			return pri_ctx;
 	}
 
@@ -3086,7 +3096,7 @@ struct ofono_gprs *ofono_gprs_create(struct ofono_modem *modem,
 	}
 
 	gprs->status = NETWORK_REGISTRATION_STATUS_UNKNOWN;
-	gprs->netreg_status = NETWORK_REGISTRATION_STATUS_UNKNOWN;
+	gprs->netreg_status = -1;
 	gprs->used_pids = l_uintset_new(MAX_CONTEXTS);
 
 	return gprs;
@@ -3097,6 +3107,7 @@ static void netreg_watch(struct ofono_atom *atom,
 				void *data)
 {
 	struct ofono_gprs *gprs = data;
+	int status;
 
 	if (cond == OFONO_ATOM_WATCH_CONDITION_UNREGISTERED) {
 		gprs_netreg_removed(gprs);
@@ -3104,7 +3115,16 @@ static void netreg_watch(struct ofono_atom *atom,
 	}
 
 	gprs->netreg = __ofono_atom_get_data(atom);
-	gprs->netreg_status = ofono_netreg_get_status(gprs->netreg);
+	status = ofono_netreg_get_status(gprs->netreg);
+
+	/*
+	 * If the status is known, assign it, otherwise keep the init value
+	 * to indicate that the netreg atom is not initialised with a known
+	 * value
+	 */
+	if (status != NETWORK_REGISTRATION_STATUS_UNKNOWN)
+		gprs->netreg_status = status;
+
 	gprs->status_watch = __ofono_netreg_add_status_watch(gprs->netreg,
 					netreg_status_changed, gprs, NULL);
 
